
/*
 * Created on Jul 27, 2005
 *
 */
package se.umu.jobsubmission.client;

/**
 * @author tordsson
 *
 * Client for testing load on JSS, concurrently submits n jobs
 * in n threads.  
 */
	
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileReader;
import java.util.Calendar;

import javax.xml.rpc.Stub;

import org.apache.axis.message.addressing.Address;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.ggf.schemas.jsdl._2005._11.jsdl.JobDefinition_Type;
import org.globus.axis.util.Util;
import org.globus.wsrf.client.ServiceURL;
import org.globus.wsrf.config.ContainerConfig;
import org.globus.wsrf.encoding.ObjectDeserializer;
import org.globus.wsrf.impl.security.authentication.Constants;
import org.xml.sax.InputSource;

import se.umu.jobsubmission.JobPreferencesType;
import se.umu.jobsubmission.JobRequest;
import se.umu.jobsubmission.JobSubmission;
import se.umu.jobsubmission.JobSubmissionServiceAddressingLocator;
import se.umu.jobsubmission.impl.jobsubmissionservice.JobSubmissionConstants;
import se.umu.jobsubmission.util.JobDescriptionTranslator;
import se.umu.jobsubmission.util.ClientUtils; 

public class LoadClient {

    static {
        Util.registerTransport(); 
    }
    
    
    public JobDefinition_Type convertJobDescription(String fileName) 
    throws Exception {
    	
    	String configFile = "/etc/jobsubmission/clientConfig"; 
    	String globusLocation = ContainerConfig.getGlobusLocation(); 
    	String converterPath = globusLocation + configFile; 
    	JobDescriptionTranslator converter = ClientUtils.getConverter(converterPath)[0];     	    
    	return (JobDefinition_Type) converter.translate(fileName); 
    }
	
    
    private class SubmitterThread extends Thread {
    	
		/**
		 * @param jobsubmissionservice
		 * @param request
		 */
		public SubmitterThread(JobSubmission jobsubmissionservice,
				JobRequest request, int i) {
			super();
			this.jobsubmissionservice = jobsubmissionservice;
			this.request = request;
			this.setName("" + i); 
		}
    	JobSubmission jobsubmissionservice; 
    	JobRequest request; 
    	/*
    	private String jobDescriptionFile; 
    	private String jobPreferencesFile; 
    	private String indexServers[]; 
    	private String URI; 
    	private String host; 
    
    	public SubmitterThread(String jobDesc, 
    			String jobPref, 
				String host, 
				String indexServers[], 
				int id) throws Exception {
    		
    		super("" + id); 
    		this.jobDescriptionFile = jobDesc; 
    		this.jobPreferencesFile = jobPref; 
    		URI = ServiceURL.getURL(
            		host, JobSubmissionConstants.SERVICE_NAME).toString(); 
    		this.indexServers = indexServers; 
    	}
    	*/
     
    	
    	public void run() {
    		try {
    			
    			// The actual job submission: 
    			System.out.println("Thread " + this.getName() + 
    					": Submitting job..."); 
    			
    			Calendar before = Calendar.getInstance(); 
    			String jobID = jobsubmissionservice.requestJob(request); 
    			Calendar after = Calendar.getInstance();     			
    			long duration = after.getTimeInMillis() - before.getTimeInMillis();                         
    			System.out.println("Thread " + this.getName() + 
    					": Job submitted with jobid " + jobID); 
    			System.out.println("Thread " + this.getName() +
    					": Call took: " + duration + " ms");     			
    		} catch (Exception e) {
    			System.err.println("Ooops:"); 
    			e.printStackTrace(); 
    		}
    		
    	}
    	
    	
    }

    
    public void submitJobs(String desc, 
    		String pref, 
			String host, 
			String indexServers[], 
			int noJobs) {
    	
    	try {    		 
    		
    		System.out.println("Preparing submission"); 
			if(desc == null) {
				System.err.println("The -p flag (JobDescription) must be specified"); 
				throw new 
				Exception("A jobDescription must be specified"); 
			}
			JobDefinition_Type jobDesc = convertJobDescription(desc); 
			
			// Locate service
			JobSubmissionServiceAddressingLocator locator = 
				new JobSubmissionServiceAddressingLocator(); 
			
			String URI = ServiceURL.getURL(
            		host, JobSubmissionConstants.SERVICE_NAME).toString(); 
			
			EndpointReferenceType endpoint = new EndpointReferenceType();
			endpoint.setAddress(new Address(URI));
			JobSubmission jobsubmissionservice = 
				locator.getJobSubmissionPort(endpoint);        
			
			// Set client security descriptor file on service stub. 
			String clientDesc = "etc/jobsubmission/client-security-config.xml"; 
			((Stub)jobsubmissionservice)._setProperty(
					Constants.CLIENT_DESCRIPTOR_FILE, 
					clientDesc);
			
			// Build job request message
			
			// Add the three parameters (index servers, job desc, job prefs)
			JobRequest requestParameters = new JobRequest(); 
			
			// Index servers
			requestParameters.setIndexServerURIs(indexServers); 
			
			// Job request: 
			requestParameters.setJobDescription(jobDesc); 
	
			// Deserialize jobpreferences from file: 
			if(pref != null) {
				
				try {
			  		JobPreferencesType preferences = null; 
	        		FileInputStream input = new FileInputStream(pref); 
	        		InputSource inputSource = new InputSource(input); 
	        		preferences = (JobPreferencesType) ObjectDeserializer.deserialize( 
	        				inputSource, JobPreferencesType.class);  
	        		requestParameters.setJobPreferences(preferences); 
				} catch (Exception e) {
					System.err.println("Could not read job preferences from file "
							+ pref); 
				}
			}
			
			// request.setRequest(requestParameters);
    		
			// Making a first call to see if that deadlock disappears...
			
			String jobID = jobsubmissionservice.requestJob(
    					requestParameters); 
			System.out.println("A first dummy-job: " + jobID); 
			
    		
    		System.out.println("Creating threads"); 
    		Thread threads[] = new Thread[noJobs];  
			for(int i = 0; i < noJobs; i++) {
				Thread t = new SubmitterThread(jobsubmissionservice, 
						requestParameters, i); 					
				threads[i] = t; 
			}
			
			for(int i = 0; i < noJobs; i++) {
				threads[i].start(); 
			}
			
			Thread.yield(); 
			
			System.out.println("Sleeping"); 
			System.out.println("Waiting for threads to complete:"); 
			for(int i = 0; i < noJobs; i++) {
				threads[i].join(); 
			}
			System.out.println("All jobs done"); 
			
			
    	} catch (Exception e) {
    		System.err.println("Ooops: "); 
			e.printStackTrace(); 
    	}
    	
    }
    
	public static void main(String args[]) {
		
		if(args.length < 4) {
			System.err.print("Usage: java LoadClient "); 
			System.err.print("jobDesc jobPref jssHost #jobs "); 
			System.err.println("[indexserver ...]"); 
			System.exit(1); 
		}
		
		try  {
			
			String jobDesc = args[0]; 
			String jobPref = args[1]; 
			String host = args[2]; 
			int numberOfJobs = Integer.parseInt(args[3]); 
			
			String indexServers[] = null; 
			if(args.length > 4) {
				indexServers = new String[args.length-4]; 
				for(int i = 4; i < args.length; i++) {
					indexServers[i-4] = args[i]; 
				}
			}
			
			LoadClient client = new LoadClient(); 
			client.submitJobs(jobDesc, 
					jobPref, 
					host, 
					indexServers, 
					numberOfJobs); 
			
		} catch (Exception e) {
			System.err.println("Ooops"); 
			e.printStackTrace(); 
		}
		
	}
	

}


