/*
 * Created on Jul 27, 2005
 *
 */
package se.umu.jobsubmission.client;

/**
 * @author tordsson
 *
 * A class for testing thoughput, submitting a job at the time, 
 * waiting with the next one until the first has completed...
 */
	
	
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileReader;
import java.util.Calendar;

import javax.xml.rpc.Stub;

import org.apache.axis.message.addressing.Address;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.ggf.schemas.jsdl._2005._11.jsdl.JobDefinition_Type;
import org.globus.axis.util.Util;
import org.globus.wsrf.client.ServiceURL;
import org.globus.wsrf.config.ContainerConfig;
import org.globus.wsrf.encoding.ObjectDeserializer;
import org.globus.wsrf.impl.security.authentication.Constants;
import org.xml.sax.InputSource;

import se.umu.jobsubmission.JobPreferencesType;
import se.umu.jobsubmission.JobRequest;
import se.umu.jobsubmission.JobSubmission;
import se.umu.jobsubmission.JobSubmissionServiceAddressingLocator;
import se.umu.jobsubmission.impl.jobsubmissionservice.JobSubmissionConstants;
import se.umu.jobsubmission.util.JobDescriptionTranslator;

public class ThroughputClient {

    static {
        Util.registerTransport(); 
    }

    private String jobDescriptionFile; 
    private String jobPreferencesFile; 
    private String indexServers[]; 
    private String URI; 
    private String host; 
    private int noJobs; 

    public ThroughputClient(String jobDesc, 
    		String jobPref, 
			String host, 
			int noJobs, 
			String indexServers[]) throws Exception {
    	
        this.jobDescriptionFile = jobDesc; 
        this.jobPreferencesFile = jobPref; 
        
        URI = ServiceURL.getURL(
        		host, JobSubmissionConstants.SERVICE_NAME).
					toString(); 
        
        this.noJobs = noJobs; 
        indexServers = null; 
        
    }
    
    
        
    public JobDescriptionTranslator getConverter(String clientConfig) {
    	String className = null; 
    	try {
    		BufferedReader in = new BufferedReader(new FileReader(clientConfig));
    		className = in.readLine().trim();
    		in.close();
    		JobDescriptionTranslator converter = (JobDescriptionTranslator) 
				Class.forName(className).newInstance(); 
    	
    		return converter; 
    	} catch (Exception e) {
    		System.err.println("Can not create converter:"); 
    		e.printStackTrace(); 
    		return null; 
    	}
    }
    
    public JobDefinition_Type convertJobDescription(String fileName) 
    throws Exception {
    	
    	String configFile = "/etc/jobsubmission/clientConfig"; 
    	String globusLocation = ContainerConfig.getGlobusLocation(); 
    	String converterPath = globusLocation + configFile; 
    	JobDescriptionTranslator converter = getConverter(converterPath); 
    	
    	return (JobDefinition_Type) converter.translate(fileName); 
    }
    
    public void submitJob() throws Exception {

        if(jobDescriptionFile == null) {
        	System.err.println("The -p flag (JobDescription) must be specified"); 
        	throw new 
				Exception("The -p flag (JobDescription) must be specified"); 
        }

        // Locate service
        JobSubmissionServiceAddressingLocator locator = 
            new JobSubmissionServiceAddressingLocator(); 
                        
        EndpointReferenceType endpoint = new EndpointReferenceType();
        endpoint.setAddress(new Address(URI));
        JobSubmission jobsubmissionservice = 
            locator.getJobSubmissionPort(endpoint);        
        
        // Set client security descriptor file on service stub. 
        String clientDesc = "etc/jobsubmission/client-security-config.xml"; 
        ((Stub)jobsubmissionservice)._setProperty(
        		Constants.CLIENT_DESCRIPTOR_FILE, 
				clientDesc);
                
        // Add the three parameters (index servers, job desc, job prefs)
        // to job request message
        JobRequest requestParameters = new JobRequest(); 

        // Index servers
        requestParameters.setIndexServerURIs(this.indexServers); 

        // Job request: 
        
        JobDefinition_Type jobDescription = convertJobDescription(
        		jobDescriptionFile); 
        requestParameters.setJobDescription(jobDescription); 
        
        // Deserialize jobpreferences from file: 
        if(this.jobPreferencesFile != null) {
        	
        	try {
          		JobPreferencesType preferences = null; 
        		FileInputStream input = new FileInputStream(this.jobPreferencesFile); 
        		InputSource inputSource = new InputSource(input); 
        		preferences = (JobPreferencesType) ObjectDeserializer.deserialize( 
        				inputSource, JobPreferencesType.class);      		
        		requestParameters.setJobPreferences(preferences); 
        		
        	} catch (Exception e) {
        		System.err.println("Could not read job preferences from file "
        				+ this.jobPreferencesFile); 
        	}
        }
    
        // The actual job submission: 

        Calendar first = Calendar.getInstance(); 
        int noFails = 0; 
        int thisFails = 0; 
        boolean done = false; 
        String jobID = null; 
        int i = 0; 
        
        for (i = 0; i < this.noJobs && thisFails < 3; i++) {
        
        	done = false; 
        	thisFails = 0; 
        	Calendar before = Calendar.getInstance(); 
        	while(!done) {
        		try {
        			jobID = jobsubmissionservice.requestJob(requestParameters); 
        			done = true; 
        		} catch (Exception e) {
        			noFails++; 
        			thisFails++; 
        			System.out.print("Submission failed, "); 
        			if(thisFails >= 3) {
        				System.out.println("cancelling"); 
        				done = true; 
        			}
        			else {
        				System.out.println("retrying"); 
        				done = false; 
        			}
        		}
        	}
        	Calendar after = Calendar.getInstance(); 
        	long duration = after.getTimeInMillis() - before.getTimeInMillis();                         
        	System.out.println("Job " + i + " submitted with jobid " 
        			+ jobID); 
        	System.out.println("Call took: " + duration + " ms"); 
        } 
        Calendar last = Calendar.getInstance(); 
        long totalTime = last.getTimeInMillis() - first.getTimeInMillis(); 
        if(i != this.noJobs -1) {
        	System.out.println("Total time for submitting " + i
        			+ " jobs = " + totalTime + " ms"); 
        	System.out.println("Average time for job = " 
        			+ totalTime / i + " ms"); 
        }
        else {
        	System.out.println("Total time for submitting " + this.noJobs
        			+ " jobs = " + totalTime + " ms"); 
        	System.out.println("Average time for job = " 
        			+ totalTime / this.noJobs + " ms"); 
        }
        System.out.println("Totally, there was " + noFails + " failed jobs");

    }
    
    
	public static void main(String args[]) {
		
		if(args.length < 4) {
			System.err.print("Usage: java ThoughputClient "); 
			System.err.print("jobDesc jobPref jssHost #jobs "); 
			System.err.println("[indexserver ...]"); 
			System.exit(1); 
		}
		
		try  {
			
			String indexServers[] = null; 
			if(args.length > 4) {
				indexServers = new String[args.length-4]; 
				for(int i = 4; i < args.length; i++) {
					indexServers[i-4] = args[i]; 
				}
			}
			
			ThroughputClient client = new ThroughputClient(args[0],
					args[1], 
					args[2], 
					Integer.parseInt(args[3]), 
					indexServers); 
			
			client.submitJob(); 
			
			
		} catch (Exception e) {
			System.err.println("Ooops"); 
			e.printStackTrace(); 
		}
		
	}
	

}


